#!/usr/bin/env python3
# -*- coding: utf-8 -*-

import sys
import math
import time
import numpy as np
import threading

sys.path.append('robot')
import Robot as rob
 
DISTANCE_THLD = 0.15 #m
 
def collect_scans(robot, filename):

    with open(filename,'w') as scanfile:

        print ( 'Started collecting' )

        while not robot.navigation_stop:

            scan = None
            pose = None

            # read scan
            while scan is None or len(scan[0]) < 1 or pose is None or np.any(np.isnan(pose)) or np.any(np.isnan(scan)):
                pose = robot.get_pose()
                scan = robot.robot.get_laser_scan()
            scan_x, scan_y = scan

            # write scan to file
            towrite = '%f,%f,%f,%s,%s\n' % (pose[0],pose[1],pose[2],','.join(['%f' % scanval for scanval in scan_x]),','.join(['%f' % scanval for scanval in scan_y]))
            scanfile.write(towrite)

            time.sleep(1.0)

        print ('Scanning done')    
 
 
if __name__=="__main__":

    #navigation points
    #route = [(6.0, 8.5, None), (4.0, 6.0, None), (5.0, 5.0, None), (1.5, 1.5, None)]
    route = [(8.5, 4.0, None), (6.0, 6.0, None), (5.0, 5.0, None), (1.5, 8.5, None)]
 
    if len(sys.argv) >= 2:
        scanfile = sys.argv[1]
    else:
        scanfile = 'test.scan'

    print ( 'Collecting scan into file %s' % scanfile)    

    #instantiate the robot
    robot = rob.Robot()
    robot.navigation_stop = False

    #run scan collection
    scan_thread = threading.Thread(target=collect_scans, args=(robot,scanfile))
    scan_thread.start()     

    #navigate
    for waypoint in route:
        pos_des = waypoint[0:2]
 
        print("Navigation point " + str(pos_des))

        #navigate the robot towards the target
        status1 = robot.goto(pos_des)
 
    #stop the robot
    robot.navigation_stop = True            
